//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
 /*
 *  USB-MIDI 4 button and 4 RGB led footswitch
 *  
 *  contains code by
 *     PJRC, see: http://www.pjrc.com/teensy/td_midi.html
 *     Rodrigo Constanzo, http://www.rodrigoconstanzo.com // rodrigo.constanzo@gmail.com
 *     Constantin Popp, constantin.popp@gmail.com
 *  
 *  EXPLANATION
 *  -----------
 *  The code checks for 4 digital pins and uses these to send MIDI CC messages to the computer.
 *  The code also listens for MIDI Note messages and uses these to set the brightness of
 *  RGB leds, one per button, allowing for decoupled LED feedback.
 *  (example Max/MSP code below)
 *
 */
//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////




#include <Bounce.h>

// define MIDI channel
const int channel = 9;

// assign led iterator to TeensyPin
byte led2pin[] = {3, 4, 5, 6, 9, 10, 20, 21, 22, 23, 32, 25};

// assign led iterator to note number
byte led2note[] = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16};

// assign note number to led iterator -> will be set automatically
byte note2led[128] = {-1};

// button assignment
Bounce button1 = Bounce(14, 55); // set debounce time
Bounce button2 = Bounce(15, 55);
Bounce button3 = Bounce(16, 55);
Bounce button4 = Bounce(17, 55);




/////////// SETUP //////////////


void setup() {
  
  // setup button pits with internal pullup resistor
  for (int i = 14; i <= 17; i++) { // these are for the buttons
    pinMode(i, INPUT_PULLUP);
  }
  
  // set led pins as outputs  // these are for the LEDS
  for (int i = 0; i <= 11; i++) {
    pinMode(led2pin[i], OUTPUT);
    note2led[led2note[i]] = i; // assign note number to led iterator
  }

  startupFade();
  
  // assign incoming MIDI note data to callback function
  usbMIDI.setHandleNoteOn(OnNoteOn);
  usbMIDI.setHandleNoteOff(OnNoteOff);
}




///////////////// LOOP ///////////////////////////

void loop() {
 
  // button update
  button1.update();
  button2.update();
  button3.update();
  button4.update();
  
  // send CC messages on button on
  if (button1.fallingEdge()) {
    usbMIDI.sendControlChange(1, 127, channel);
  }
  if (button2.fallingEdge()) {
    usbMIDI.sendControlChange(2, 127, channel);
  }
  if (button3.fallingEdge()) {
    usbMIDI.sendControlChange(3, 127, channel);
  }
  if (button4.fallingEdge()) {
    usbMIDI.sendControlChange(4, 127, channel);
  }
  
  
  // send CC messages on button off
  if (button1.risingEdge()) {
    usbMIDI.sendControlChange(1, 0, channel);
  }
  if (button2.risingEdge()) {
    usbMIDI.sendControlChange(2, 0, channel);
  }
  if (button3.risingEdge()) {
    usbMIDI.sendControlChange(3, 0, channel);
  }
  if (button4.risingEdge()) {
    usbMIDI.sendControlChange(4, 0, channel);
  }

  // update MIDI
  usbMIDI.read();
}




//////////// FUNCTIONS //////////////////////


// note handling
void OnNoteOn(byte channel, byte note, byte velocity) {
  analogWrite(led2pin[note2led[note]], velocity * 2);
}

void OnNoteOff(byte channel, byte note, byte velocity) {
  analogWrite(led2pin[note2led[note]], 0);
}


void startupFade() {

  // store RED, GREEN, BLUE LEDs for startup fade
  byte fadeOrder[] = {0, 4, 8};
  
  // store LEDs for white startup fade
  byte whiteLED[] = {9, 10, 11};
  
  // define fade delay
  int fadeDelay = 8;
  
  
  // fade in first three LEDs (RGB)
  for (int i = 0; i <= 2; i++) {
    for (int fadeValue = 0; fadeValue <= 250; fadeValue += 25) {
      analogWrite(led2pin[fadeOrder[i]], fadeValue);
      delay(fadeDelay);
    }
    for (int fadeValue = 250; fadeValue >= 0; fadeValue -= 25) {
      analogWrite(led2pin[fadeOrder[i]], fadeValue);
      delay(fadeDelay);
    }
  }
    
    
  // fade in fourth LED (white)
  for (int fadeValue = 0; fadeValue <= 250; fadeValue += 25) {
    for (int i = 0; i <= 2; i++) {
      analogWrite(led2pin[whiteLED[i]], fadeValue);
    }
    delay(fadeDelay);
  }
    
  for (int fadeValue = 250; fadeValue >= 0; fadeValue -= 25) {
    for (int i = 0; i <= 2; i++) { 
      analogWrite(led2pin[whiteLED[i]], fadeValue);
    }
    delay(fadeDelay);
  }
  
}




//////////// EXAMPLE MAX CODE //////////////////////
/*

----------begin_max5_patcher----------
2891.3oc6cs0aqiaD94jeEBF6SEodEuS0GJ5ksXwBz8oVfhhytHP1lwQmirj
gj74jzE6+8RJJYKYaIQEayDmvbvZ3kTWluYFNyvOQQ+a2dyjYoOIxm38m79j
2M27a2dyMkMoZ3lp++alrJ7o4wg4kG1j4oqVIRJlbmtuBwSEksuP7P3l3Bu4
oKDdaxE4dyeLLIQD6ETerwQIh4oaRJOAXUiqCKl+XTxx6yDyKzBBH.O0+NO.
ELE3K+idmW.T0BBM026WqNwnEk22zYe9OxquCIaVEkDKJJEUPUi4EOGKJO1F
GV5lh5iyW03ue6spOtyPsPh3axa7AJgL4kUrCvGAaA9ZnQmRqgFS0BDdbnQO
JzfFBs5iS2TwyqEZoPd3x+y6WeAHuK6e1xYRaeRQVZbOvmJMjDkgjp.MUCc+
iCcF4shYcSxmSiR7PCCLLis0vBnn9rrL5IAObul16NESbGZguJdZcl22EA79
dOn2eIedXbX1J0vcfAJFDeJJP9GdXEC6jTLfNULuHkQ92TXx.WZZ0H6RrICc
AkdA.3QAHe2kKKbknPjcuHIblFV9U8IGnIuaQyhhiJdtIvZoUPieb+CwogEu
LkwJQdd3RwAtFRb9c84Dv3bkNh.nS4R0BNP5P.50GH3jh6cl8A5D1fAfM1+.
X62Gr49WEv1ueXWMhvbqMG7VB1mVFNhOx3LbT1UTFtJfMhLbT9GhLb0JFyyv
QCtpxvU6ROhLb9u2xvEzeHORPv9g7P3dSvAtFhzyG.0DvHQM5Z.0rAPsd3vH
PM98S5MD.ad5M+qnzaU.aLo2.eHRuUqXFQ5M3UU5sZWZySuQQu2RuQ6OjGF3
OtPdT70PfdR+nFQgiD0jqATiG.05gCi.0zqhzahEdoIee5COLV5ngUw+.S4U
w+PA8wFMI3sRZu+fG.x5ieB+8S4oQVWVZxEx+NJ4EFNqHc4xXwvHTVsRctKn
1WF2ABwlDY+7lV6Ei9SqRN.jXbkbDn0co6BbEod+y+wOjOLvH7RfAPkTOg6x
kdaMpOH0XIRSdYW+0rnv3QhY0E3gv4hlGkps7n+WYaH0yv50uP2J0yHJzEG7
gnP2ZEi4E5hoWUE5VOh27Bcwj2aE5h5u3GHbjLZfuJJzcfGOCfMRFMvWE73L
vSmoNOg4nF9VB0cD6KIsPHuCd+xj+sPjj+r2O+S+vO8KS5cQITqIHUYL6sHP
zwyFfre4sq8lsonHu24xv0U6QJqAn2UZA5zXqpy.YyBSVpBloKyqorJxpvbE
nkkKDEK9pHKOJMow89lIgqW2n4aZbJJM0mSKuPr611TThtI31lxDeMp97waa
MLSpwJjpqMYZr8DuNkl5xHyXlkrIpTTzMJsY2VeA2w8coWie0XG8S5i1nRRo
4cYb57uHVzHqgTSsVjDkrNSjKqySl+PKba6tZAEceyRn.aKgpc+0kcczNOZQ
c2LYYVzhzDkPzRUqZt91IGYT53n9bGXJOhjv0G4jk9QR0RGclKA4l7YgYJKQ
UJzZCjrr1z31cs87hEOTT085njj8zhEoq6tyrnkO1y4NKU14p9t1k8je+lDc
u2KGRVbed3WaqsKBiiqFi19x+TXRzpvBQQj1DHqzntScYDOlOWNKk3V3U2yW
OROKjNwyEeKZQwik2nlNCxCOZcsSzjsV4EQKE4EsaqHbYd6VNXvsroMypFjd
egX05XIJZe.sV4bMGQ1LZVq16cFNsCsoCZrqiilzVOiM7TFWNbiKydcvb3NH
CVyq4wBy0g1niv32TGJuJzvEPQzZF9cpJP780EXzP5B3aRcQqTcsUEp0S4xA
UE9kqTRYAcAn..BIypevJNYeMAfe1TEnF8bzR5UYAuqImGWd8VtHd67957Y5
w0w4YS4H4LiUClnCp1Xcp1fiTsA6SsUW.g0TX+YoWjuY9Y.YpQNmFPQGaM9b
fFid1zXf9zXmcuqR4zLMBrrhAje8mcF9A0otv+bpKlzR.pC0VdAOiJH8s0rA
YAA6xXMrZ5LlxpC8P43KaqKpG+LNkA4hqLpx2dD0wtBfUO3jNp7oDop9OtdJ
OcS17ZOy5HBdswqrTthnjsyG3S6f9dG3iQKVHR1GLKhxUkOtv7DyiVlYlJy3
2LxrJZiQxrpVf2J5YS8MNhA4USl4lJyzyjLWkmqdh4STSNaw85IhdeXQQVzr
ME5gqMYZXTSXRN68YgwUSGZ6zo6a1S6lh0s6j2Qx5ihbGozML6NbbEE1C8z8
PfKCe8kYPtLO.SDlNRLBuhdBluXh8fXn4D6gbD64H1yQrmiXOGwdNh8bD64H
1yQrmiXOGwdNh8bD6c4H1y5Dg7wjvoqRh8tBIP09DU+AlXuZ1cLmzK7UGwdP
l+HwH4i.wdpcFISI1CxbD64H1yQrmiXOGwdNh8bD64H1yQrmiXOGwdNh8bD6
4H1yQrmiXOGwdusH1qlcGiI8Bxu5H1qZ2KeDXL3cvdNxCYoq7z1+7gUN.e1T
rZymPob748sAcvfuq1fNFlAT8NDuYDfRcDf5H.0Q.pi.TGAnNBPcDf5H.0Q.
pi.TGAnNBPcDf5dkkcuxxNhlcuxxusH.MXjK5OH75i+yw9VYC8eGrvFCm+Eu
H0+F929Q3NtO6+29Q.4j1oEsy9M4rMKV7bua3vZXy4SApcUW1fnFeRnFM3tM
74D85evOAdPOjGd3w89jxsZTe4TOPvd2ZgO93dhgJAhQa4xmakw7BYZmAUB5
YoNfOvIE0C02n9iMg9J3WJIsKUTqM1OkZkN4fTopMH1cYH6HMpZ+SswAcPJz
CSedf0wX4AcNjmUQKVmJUVUJE.uzwkfoSCTQwTaS1HR6l1cKOeXAeAvBTu4v
aarPoW.rfzYYsNVHWBrvfuJX4R3igg9uFXgcIvBA7pL1mcIhiQnuJi8U+v+c
1wB0O30.K7KAVX9uJi8U+z6d9wB40Yrevk.KA7WCrXhKlZJCdPqTRkRb.lHN
.qHNpU.xfxCz2ZUbBMoxFHzdxiIhC0dhCyD4gaW4YP2m.qZtFTbXVSbT+jPL
n7ff1UdFLtNvdxiQ0kgsq7Ln8hXU2Y3PhCxphCZHwwd4JvlLyQr8BNiMI2E1
hr2XRUbXnckmgFcgQ1UdFZ3E1dQeHlLiah8h9PHmC+m8lE.WOgFe1TV0C2od
R.aa5RPLCv.SM0dlZpISCfRrq7LjolZuPoTSp6lZuBUnlTnB0dgRYlTHG0do
ZnljpgZu4kr2s53xCCaW4Ynw6LjckmghGxr23KF0D4IvtxyP1Ktuckmg7m4V
zdYBODL6kufYz3c6MwefIkxAH1Ud.lHO.6IOPSjGKwJqIk5ZwxeLgkO647Xh
s5klKUulI16k8SII68R9s2K32gubec+h8s+K0W45FTuHP1acZr60layhnz+U
4Ku18+rHYidchT+J10VeMa4CQwwySiSO3cardoiLQ2a8JEb6w9IO+ov.L.vk
JuoH.hAokeS9EBowfwpyATeRXBNPs9ujeiBYXR423PDAo9l+dmFb28xGDnuC
9Aber9axl.x6UySKLYY0KqGqwab15rz0oYaeqEmhB1d7aJRWlEtHpZMU62ZI
PdWk6Vlr28W6MSVIUnQ0mSS2ysFiujGGIMf+Gk0sOCQC0Z4Sls0G0KTmSTVz
qbo+V7FQ8ZL53xR374xaPKCMBvnXlRkS2Z8BTMAt.BXV52RFsDtStHR+O0um
0Ro9BIg+8mCGu.5CCH3R+WFCgfkJSND..nyu.9iYBwKPBK0dH0epuAnbpRLO
2B2+UDGm9sQKcbHATpzXb.GTFpfH0njVi9a8Js+ox2+XRfuO+bAfMqlIx9wz
3EiV7Uth9kleR.DPJQBnY3qyh.loC2njv9C6ONY6taOL1OESonfJuXFS6OKs
GAub+Y447629+AjmcQEB
-----------end_max5_patcher-----------

*/
